@@ -262,7 +262,8 @@ Get application from L<Minion/"app">.
   my $err = $job->execute;
 
 Perform job in this process and return C<undef> if the task was successful or an
-exception otherwise.
+exception otherwise.  Note that this method should only be used to implement
+custom workers.
 
   # Perform job in foreground
   if (my $err = $job->execute) { $job->fail($err) }
@@ -419,22 +420,25 @@ Id of worker that is processing the job.
 
   my $bool = $job->is_finished;
 
-Check if job performed with L</"start"> is finished.
+Check if job performed with L</"start"> is finished. Note that this method
+should only be used to implement custom workers.
 
 =head2 kill
 
   $job->kill('INT');
 
-Send a signal to job performed with L</"start">.
+Send a signal to job performed with L</"start">. Note that this method should
+only be used to implement custom workers.
 
 =head2 note
 
   my $bool = $job->note(mojo => 'rocks', minion => 'too');
 
-Change one or more metadata fields for this job. The new values will get
-serialized by L<Minion/"backend"> (often with L<Mojo::JSON>), so you shouldn't
-send objects and be careful with binary data, nested data structures with hash
-and array references are fine though.
+Change one or more metadata fields for this job. Setting a value to C<undef>
+will remove the field. The new values will get serialized by L<Minion/"backend">
+(often with L<Mojo::JSON>), so you shouldn't send objects and be careful with
+binary data, nested data structures with hash and array references are fine
+though.
 
   # Share progress information
   $job->note(progress => 95);
@@ -446,13 +450,15 @@ and array references are fine though.
 
   $job->perform;
 
-Perform job in new process and wait for it to finish.
+Perform job in new process and wait for it to finish. Note that this method
+should only be used to implement custom workers.
 
 =head2 pid
 
   my $pid = $job->pid;
 
-Process id of the process spawned by L</"start"> if available.
+Process id of the process spawned by L</"start"> if available. Note that this
+method should only be used to implement custom workers.
 
 =head2 remove
 
@@ -508,7 +514,8 @@ Queue to put job in.
 
   $job = $job->start;
 
-Perform job in new process, but do not wait for it to finish.
+Perform job in new process, but do not wait for it to finish. Note that this
+method should only be used to implement custom workers.
 
   # Perform two jobs concurrently
   $job1->start;
@@ -521,7 +528,8 @@ Perform job in new process, but do not wait for it to finish.
 
   $job->stop;
 
-Stop job performed with L</"start"> immediately.
+Stop job performed with L</"start"> immediately. Note that this method should
+only be used to implement custom workers.
 
 =head1 SEE ALSO
 
