@@ -7,11 +7,16 @@ function checkAll() {
   });
 }
 
-function fromNow() {
+function humanTime() {
   $('.from-now').each(function () {
     var date = $(this);
     date.text(moment(date.text() * 1000).fromNow());
   });
+  $('.duration').each(function () {
+    var date = $(this);
+    console.log(date.text() * 1000);
+    date.text(moment.duration(date.text() * 1000).humanize());
+  });
  }
 
 function pageStats(data) {}
